@@ -162,6 +162,7 @@ struct prop_info {
 	STRING_PROP   (E_CONTACT_HOMEPAGE_URL,       "wWWHomePage"),
 	STRING_PROP   (E_CONTACT_FREEBUSY_URL,       "msExchFBURL"),
 	STRING_PROP   (E_CONTACT_NOTE,               "info"), 
+	STRING_PROP   (E_CONTACT_FILE_AS,            "fileAs"),
 
 #undef STRING_PROP
 #undef COMPLEX_PROP
@@ -766,6 +767,7 @@ func_contains(ESExp *f, int argc, ESExpResult **argv, void *data)
 
 	propname = argv[0]->value.string;
 	str = argv[1]->value.string;
+	ldap_attr = query_prop_to_ldap(argv[0]->value.string);
 
 	if (!strcmp(propname, "x-evolution-any-field")) {
 		/* This gui does (contains "x-evolution-any-field" ""),
@@ -776,12 +778,15 @@ func_contains(ESExp *f, int argc, ESExpResult **argv, void *data)
 		if (strlen(str) == 0) {
 			r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 			r->value.bool = FALSE;
+		} else if (!strcmp(propname, "file_as")) {
+			r = e_sexp_result_new(f, ESEXP_RES_STRING);
+			r->value.string = g_strdup_printf ("(|(displayName=%s*)(sn=%s*)(%s=%s*))", 
+							   str, str, ldap_attr, str);
 		} else
 			r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 		return r;
 	}
 
-	ldap_attr = query_prop_to_ldap(argv[0]->value.string);
 	if (!ldap_attr) {
 		/* Attribute doesn't exist, so it can't possibly match */
 		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
