@@ -66,7 +66,7 @@ struct _ExchangeAccountPrivate {
 	gboolean uris_use_email;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *ad_server;
+	char *username, *password, *windows_domain, *ad_server;
 	E2kAutoconfigAuthPref auth_pref;
 	int ad_limit;
 
@@ -265,6 +265,9 @@ finalize (GObject *object)
 			strlen (account->priv->password));
 		g_free (account->priv->password);
 	}
+	if (account->priv->windows_domain)
+		g_free (account->priv->windows_domain);
+
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
@@ -959,6 +962,7 @@ exchange_account_connect (ExchangeAccount *account)
 	struct dirent *dent;
 	DIR *d;
 	char *old_password, *new_password;
+	char *user_name = NULL;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
@@ -975,10 +979,18 @@ exchange_account_connect (ExchangeAccount *account)
 	account->priv->connecting = TRUE;
 	g_mutex_unlock (account->priv->connect_lock);
 
+	if (account->priv->windows_domain)
+		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, 
+							account->priv->username);
+	else
+		user_name = g_strdup (account->priv->username);
+
 	ac = e2k_autoconfig_new (account->home_uri,
-				 account->priv->username,
+				 user_name,
 				 NULL,
 				 account->priv->auth_pref);
+	g_free (user_name);
+
 	e2k_autoconfig_set_gc_server (ac, account->priv->ad_server,
 				      account->priv->ad_limit);
 
@@ -1010,7 +1022,6 @@ exchange_account_connect (ExchangeAccount *account)
 
 		switch (result) {
 		case E2K_AUTOCONFIG_AUTH_ERROR:
-		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
 			errmsg = _("Could not authenticate to server. "
 				   "(Password incorrect?)\n\n");
 			goto try_password_again;
@@ -1026,13 +1037,13 @@ exchange_account_connect (ExchangeAccount *account)
 				   "password wrong.\n\n");
 			goto try_password_again;
 
+		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
+			ac->use_ntlm = 1;
+			goto try_connect_again;
+
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_BASIC:
-			errmsg = _("Could not authenticate to server."
-				   "\n\nYou may need to use Plaintext "
-				   "Password authentication.\n\nOr "
-				   "you might have just typed your "
-				   "password wrong.\n\n");
-			goto try_password_again;
+			ac->use_ntlm = 0;
+			goto try_connect_again;
 
 		case E2K_AUTOCONFIG_REDIRECT:
 			if (!redirected && account_moved (account, ac))
@@ -1508,6 +1519,12 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->priv->password_key = g_strdup_printf ("exchange://%s", account->priv->uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
+
+	if (uri->domain)
+		account->priv->windows_domain = g_strdup (uri->domain);
+	else
+		account->priv->windows_domain = NULL;
+
 	account->exchange_server = g_strdup (uri->host);
 	if (uri->authmech && !strcmp (uri->authmech, "Basic"))
 		account->priv->auth_pref = E2K_AUTOCONFIG_USE_BASIC;
