@@ -165,6 +165,13 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 	const char *internal_uri = e_folder_exchange_get_internal_uri (folder);
 	char *mf_path;
 
+	/* This should ideally not be needed. But, this causes a problem when the
+	server has identical folder names [ internal_uri ] for folders. Very much
+	possible in the case of favorite folders */
+	if (g_hash_table_lookup (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
+				(char *)e2k_uri_path (internal_uri)))
+		return;
+
 	g_hash_table_insert (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
 			     (char *)e2k_uri_path (internal_uri), folder);
 
@@ -342,9 +349,6 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	E2kHTTPStatus status;
 	EFolder *dest;
 	char *permanent_url = NULL, *physical_uri;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type = NULL;
 	ExchangeAccountFolderResult ret_code;
@@ -400,10 +404,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 		    (strcmp (folder_type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_cal);
-			remove_esource (hier->account, conf_key_cal,
+						CONF_KEY_CAL);
+			remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER,
 					physical_uri, &cal_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -411,10 +415,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "tasks/public") == 0)){
 			task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_tasks);
-			remove_esource (hier->account, conf_key_tasks,
+						CONF_KEY_TASKS);
+			remove_esource (hier->account, EXCHANGE_TASKS_FOLDER,
 					physical_uri, &task_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -422,10 +426,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "contacts/public") == 0)) {
 			cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_contacts);
-			remove_esource (hier->account, conf_key_contacts,
+						CONF_KEY_CONTACTS);
+			remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER,
 					physical_uri, &cont_source_list, 
-					FALSE, TRUE);
+					FALSE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
